indexing

	description: "Describes a single <!ELEMENT> definition."

	author:     "Berend de Boer <berend@pobox.com>"
	copyright:  "Copyright (c) 2001 by Berend de Boer, see forum.txt"
	date:       "$Date: $"
	revision:   "$Revision: $"


deferred class

	DEFINED_ELEMENT


inherit

	XM_HELPER_ROUTINES
		export
			{NONE} all
		end


feature {NONE} -- Initialization

	make (a_name: STRING) is
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		do
			name := a_name
			create attributes.make (32)
			set_default_attributes
		end

	make_ns (a_namespace, a_name: STRING) is
			-- Create element in optional name space.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		local
			full_name: STRING
		do
			if a_namespace = Void or else a_namespace.is_empty then
				full_name := a_name
			else
				create full_name.make (a_namespace.count + 1 + a_name.count)
				full_name.append_string (a_namespace)
				full_name.append_character (':')
				full_name.append_string (a_name)
			end
			make (full_name)
		end


feature -- Access

	name: STRING


feature -- Callback support

	on_end_tag: PROCEDURE [ANY, TUPLE]
			-- A client can set this to immediately call the procedure
			-- which handles a start of this tag.

	on_start_tag: PROCEDURE [ANY, TUPLE]
			-- A client can set this to immediately call the procedure
			-- which handles a start of this tag.

	on_start_tag_finish: PROCEDURE [ANY, TUPLE]
			-- A client can set this to immediately call the procedure
			-- which handles a finish of the start of this tag.

	set_on_end_tag (a_value: PROCEDURE [ANY, TUPLE]) is
			-- Set `on_end_tag'.
		do
			on_end_tag := a_value
		ensure
			end_tag_set: on_end_tag = a_value
		end

	set_on_start_tag (a_value: PROCEDURE [ANY, TUPLE]) is
			-- Set `on_start_tag'.
		do
			on_start_tag := a_value
		ensure
			start_tag_set: on_start_tag = a_value
		end

	set_on_start_tag_finish (a_value: PROCEDURE [ANY, TUPLE]) is
			-- Set `on_start_tag_finish'.
		do
			on_start_tag_finish := a_value
		ensure
			start_tag_finish_set: on_start_tag_finish = a_value
		end


feature -- Attributes

	add_attribute (a_attribute: DEFINED_ATTRIBUTE) is
			-- The only supported way of extending `attributes'.
		require
			attribute_not_void: a_attribute /= Void
		do
			attributes.put (a_attribute, as_utf8_string (a_attribute.name))
		ensure
			-- does not work due to is_equal bomb
			--attribute_added: attributes.has (a_attribute.name)
		end

	attributes: DS_HASH_TABLE [DEFINED_ATTRIBUTE, UC_UTF8_STRING]

	get_attribute (attribute_name: STRING): DEFINED_ATTRIBUTE is
			-- Return attribute if it exists in `attributes'.
		require
			attribute_name_not_empty: attribute_name /= Void and then not attribute_name.is_empty
		do
			attributes.search (as_utf8_string (attribute_name))
			if attributes.found then
				Result := attributes.found_item
			end
		end

	get_ns_attribute (a_name_space, a_local_part: STRING): DEFINED_ATTRIBUTE is
			-- Return attribute if it exists in `attributes'.
		require
			local_part_name_not_empty: a_local_part /= Void and then not a_local_part.is_empty
		local
			attribute_name: STRING
		do
			attribute_name := resolved_name (a_name_space, a_local_part)
			Result := get_attribute (attribute_name)
		end

	is_defined_attribute (attribute_name: STRING): BOOLEAN is
			-- True if `attribute_name' occurs in `attributes'.
		require
			attribute_name_not_empty: attribute_name /= Void and then not attribute_name.is_empty
		do
			attributes.search (as_utf8_string (attribute_name))
			Result := attributes.found
		end


feature {XM_VALIDATING_PARSER} -- Check tag transition

	is_valid_child (
			state: ELEMENT_STATE;
			next_element: DEFINED_ELEMENT): BOOLEAN is
			-- we come here when either we have returned True in the past,
			-- or a parent asks us what we think
		require
			this_is_my_state: state.element = Current
		deferred
		end

	is_valid_end (state: ELEMENT_STATE): BOOLEAN is
			-- return True if there are no required children that we must
			-- see, so tag may end
		require
			this_is_my_state: state.element = Current
		deferred
		end

	is_data_allowed: BOOLEAN is
			-- does this element allow data?
		deferred
		end

	is_valid_data (data: STRING): BOOLEAN is
			-- Is the data in this element valid?
		require
			data_allowed: is_data_allowed
		deferred
		end


feature {ELEMENT_STATE}

	init_element_state (state: ELEMENT_STATE) is
			-- chance to initialize state so is_valid_child can start
			-- with the first child
		require
			valid_state: state /= Void
		do
		end

feature {NONE} -- Implementation

	set_default_attributes is
			-- Attributes that may occur in every element.
		local
			xmlspace: DEFINED_ATTRIBUTE
			xmllang: DEFINED_ATTRIBUTE
		do
			create xmlspace.make ("xml:space", False, "(default|preserve)")
			add_attribute (xmlspace)
			create xmllang.make ("xml:lang", False, "([a-zA-Z][a-zA-Z]|[iI]\-[a-zA-z]+|[xX]\-[a-zA-Z]+)(\-[a-zA-Z]+)?")
			add_attribute (xmllang)
		end


invariant

	name_not_empty: name /= Void and then not name.is_empty
	attributes_not_void: attributes /= Void

end
